Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.Configuration
Imports System.Data
Imports System.Windows
Imports System.ServiceModel
Imports System.ServiceModel.Channels
Imports Contracts
Imports System.Threading
Imports System.ServiceModel.Description
Imports System.Workflow.Runtime
Imports System.Workflow.Runtime.Hosting

Namespace HostApplication
    ''' <summary>
    ''' Interaction logic for App.xaml
    ''' </summary>
    Partial Public Class App
        Inherits Application
        Friend localWorkflowHost As LocalWorkflowServiceHost(Of WorkflowsAndActivities.OrderWorkflow, HostService)

        Private workflowHost As WorkflowServiceHost
        Friend appHost As ServiceHost

        Friend LocalBinding As System.ServiceModel.Channels.Binding = New NetNamedPipeContextBinding(NetNamedPipeSecurityMode.None)
        Friend WFAddress As String = "net.pipe://localhost/OrderWF"
        Friend HostAddress As String = "net.pipe://localhost/WPFHostApp"
        Protected Overrides Sub OnStartup(ByVal e As StartupEventArgs)
            MyBase.OnStartup(e)
            ConfigureHosts()
        End Sub

        Protected Overrides Sub OnExit(ByVal e As ExitEventArgs)
            'combined custom host to encapsulate details
            If localWorkflowHost IsNot Nothing Then
                If localWorkflowHost.State = CommunicationState.Opened Then
                    localWorkflowHost.Close()
                Else
                    localWorkflowHost.Abort()
                End If
            End If

            'hosts for simple examples (window1.xaml and window2.xaml)
            If workflowHost IsNot Nothing Then
                If workflowHost.State = CommunicationState.Opened Then
                    workflowHost.Close()
                Else
                    workflowHost.Abort()
                End If
            End If

            If appHost IsNot Nothing Then
                If appHost.State = CommunicationState.Opened Then
                    appHost.Close()
                Else
                    appHost.Abort()
                End If
            End If

            MyBase.OnExit(e)
        End Sub


        Private Sub ConfigureHosts()
            'host the workflow
            'add the client endpoint for the host service
            'by using the channel manager, no app.config is required
            ThreadPool.QueueUserWorkItem(Function(o) ConfigureHostsAsync())

            'host the local service
            appHost = New ServiceHost(New HostService())
            appHost.AddServiceEndpoint("Contracts.IHostInterface", LocalBinding, HostAddress)

            Try
                appHost.Open()
            Catch ex As Exception
                appHost.Abort()
                MessageBox.Show(String.Format("There was an error hosting the local service: {0}", ex.Message))
            End Try
        End Sub

        Private Function ConfigureHostsAsync() As Object
            localWorkflowHost = New LocalWorkflowServiceHost(Of WorkflowsAndActivities.OrderWorkflow, HostService)()
            localWorkflowHost.Description.Behaviors.Find(Of ServiceDebugBehavior)().IncludeExceptionDetailInFaults = True
            localWorkflowHost.Open()
            workflowHost = New WorkflowServiceHost(GetType(WorkflowsAndActivities.OrderWorkflow))
            workflowHost.AddServiceEndpoint("Contracts.IWorkflowInterface", LocalBinding, WFAddress)
            Dim endpoint As ServiceEndpoint = New ServiceEndpoint(ContractDescription.GetContract(GetType(Contracts.IHostInterface)), LocalBinding, New EndpointAddress(HostAddress))
            endpoint.Name = "HostEndpoint-Contracts.IHostInterface"
            Dim rt As WorkflowRuntime = workflowHost.Description.Behaviors.Find(Of WorkflowRuntimeBehavior)().WorkflowRuntime()
            Dim endpoints = New List(Of ServiceEndpoint)
            endpoints.Add(endpoint)
            Dim chanMan As ChannelManagerService = New ChannelManagerService(endpoints)
            rt.AddService(chanMan)
            Try
                workflowHost.Open()
            Catch ex As Exception
                workflowHost.Abort()
                MessageBox.Show(String.Format("There was an error hosting the workflow as a service: {0}", ex.Message))
            End Try
            Return Nothing
        End Function

    End Class
End Namespace
