Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports System.ServiceModel
Imports System.IO
Imports System.Workflow.ComponentModel.Compiler
Imports System.ServiceModel.Channels
Imports System.ServiceModel.Description
Imports System.Reflection
Imports System.Workflow.Runtime
Imports System.Workflow.Runtime.Hosting
Imports Contracts
Imports System.Workflow.ComponentModel
Imports System.Workflow.Activities

Namespace HostApplication
	Public Class LocalWorkflowServiceHost(Of TWorkflow, THostService)
		Inherits WorkflowServiceHost
		#Region "Constructors "

		Shared Sub New()
			LocalBinding = New NetNamedPipeContextBinding(NetNamedPipeSecurityMode.None)
			workflowBaseAddress = "net.pipe://localhost/WFLocal/Workflow/"
			hostServiceAddress = "net.pipe://localhost/WFLocal/Host/"

		End Sub
		Public Sub New()
			MyBase.New(GetType(TWorkflow))
			Me.workflowServiceType = GetType(TWorkflow)
			Me.hostServiceType = GetType(THostService)
		End Sub

		#End Region ' Constructors


		#Region "Private Fields"

		Private workflowServiceType As Type
		Private hostServiceType As Type
		Private hostService As ServiceHost
		Private Shared LocalBinding As Binding
		Private Shared workflowBaseAddress As String
		Private Shared hostServiceAddress As String
		Private factories As Dictionary(Of Type, ChannelFactory)
		Private proxies As Dictionary(Of Type, IClientChannel)

		#End Region ' Private Fields

		#Region "Create endpoints and host service"

		Protected Overrides Function CreateDescription(<System.Runtime.InteropServices.Out()> ByRef implementedContracts As IDictionary(Of String, ContractDescription)) As ServiceDescription
			Dim desc As ServiceDescription = MyBase.CreateDescription(implementedContracts)

			'add endpoints for the workflow
			For Each contract In implementedContracts
				desc.Endpoints.Add(New ServiceEndpoint(contract.Value, LocalBinding, GetEndpointAddress(contract.Value.ContractType)))
			Next contract

			Return desc
		End Function


		Private Function GetEndpointAddress(ByVal contractType As Type) As EndpointAddress
			Return New EndpointAddress(String.Concat(workflowBaseAddress, contractType.FullName))
		End Function

		Private Function GetContractTypes(ByVal type As Type) As IEnumerable(Of Type)
			Dim contractTypes As List(Of Type) = New List(Of Type)()

			If IsServiceContractType(type) Then
				contractTypes.Add(type)
			End If
			contractTypes.AddRange(type.FindInterfaces(Function(t,o) IsServiceContractType(t), Nothing))

			Return contractTypes
		End Function

		Private Function IsServiceContractType(ByVal type As Type) As Boolean
			If type Is Nothing Then
				Return False
			End If

			Dim sContractAtts() As Object = type.GetCustomAttributes(GetType(ServiceContractAttribute), False)
			Return sContractAtts IsNot Nothing AndAlso sContractAtts.Length > 0
		End Function


		Protected Overrides Sub OnOpened()
			MyBase.OnOpened()
			hostService = New ServiceHost(hostServiceType)

			For Each contractType As Type In GetContractTypes(hostServiceType)
				hostService.AddServiceEndpoint(contractType, LocalBinding, GetHostEndpointAddress(contractType))
			Next contractType

			hostService.Open()

			'add client endpoints for the service
			AddClientEndpointsToServiceChannelManager()
		End Sub

		Private Sub AddClientEndpointsToServiceChannelManager()
			Dim endpoints As List(Of ServiceEndpoint) = New List(Of ServiceEndpoint)()

			For Each contractType As Type In GetContractTypes(hostServiceType)

				Dim endpoint As New ServiceEndpoint(ContractDescription.GetContract(contractType), LocalBinding, New EndpointAddress(GetHostEndpointAddress(contractType)))
				endpoint.Name = "HostEndpoint-" & contractType.FullName
				endpoints.Add(endpoint)
			Next contractType

			'by using the channel manager, no app.config is required
			Dim runtime As WorkflowRuntime = Me.Description.Behaviors.Find(Of WorkflowRuntimeBehavior)().WorkflowRuntime
			Dim chanMan As New ChannelManagerService(endpoints)

			runtime.AddService(chanMan)
		End Sub

		Private Function GetHostEndpointAddress(ByVal contractType As Type) As String
			Return String.Concat(hostServiceAddress, contractType.FullName)
		End Function

		#End Region ' Open ServiceHost for host service

		Protected Overrides Sub OnClose(ByVal timeout As TimeSpan)
			MyBase.OnClose(timeout)
			If hostService IsNot Nothing Then
				If hostService.State = CommunicationState.Opened Then
					hostService.Close(timeout)
				Else
					hostService.Abort()
				End If
			End If
		End Sub

		#Region "Helper Methods for consumers"

		Private privateHostServiceInstance As THostService
		Public Property HostServiceInstance() As THostService
			Get
				Return privateHostServiceInstance
			End Get
			Private Set(ByVal value As THostService)
				privateHostServiceInstance = value
			End Set
		End Property

		Public Function GetProxy(Of TServiceContract)() As TServiceContract
			Dim factory As ChannelFactory(Of TServiceContract) = GetChannelFactory(Of TServiceContract)()

			Dim proxy As TServiceContract = factory.CreateChannel()
			CType(proxy, IClientChannel).Open()
			Return proxy
		End Function

		Public Function GetWorkflowInstanceID(ByVal proxy As IClientChannel) As String
			Dim mgr As IContextManager = (CType(proxy, IClientChannel)).GetProperty(Of IContextManager)()

			Return mgr.GetContext()("instanceId")
		End Function

		Private Function GetChannelFactory(Of TServiceContract)() As ChannelFactory(Of TServiceContract)
			Dim factory As ChannelFactory(Of TServiceContract) = Nothing

			If factories Is Nothing Then
				factories = New Dictionary(Of Type, ChannelFactory)()
			End If

			If (Not factories.ContainsKey(GetType(TServiceContract))) Then
				factory = New ChannelFactory(Of TServiceContract)(LocalBinding, GetEndpointAddress(GetType(TServiceContract)))
				factory.Open()
				factories.Add(GetType(TServiceContract), factory)
			Else
				factory = CType(factories(GetType(TServiceContract)), ChannelFactory(Of TServiceContract))
			End If
			Return factory
		End Function

		Public Function GetProxy(Of TServiceContract)(ByVal instanceID As Guid) As TServiceContract
			If proxies Is Nothing Then
				proxies = New Dictionary(Of Type, IClientChannel)()
			End If

			If proxies.ContainsKey(GetType(TServiceContract)) Then
				Return CType(proxies(GetType(TServiceContract)), TServiceContract)
			Else
				Dim factory As ChannelFactory(Of TServiceContract) = GetChannelFactory(Of TServiceContract)()
				Dim proxy As TServiceContract = factory.CreateChannel()

				Dim mgr As IContextManager = (CType(proxy, IClientChannel)).GetProperty(Of IContextManager)()
				mgr.Enabled = False
				CType(proxy, IClientChannel).Open()

				proxies.Add(GetType(TServiceContract), CType(proxy, IClientChannel))

				Return proxy
			End If
		End Function

		Public Sub PerformClientOperationWithContext(ByVal workflowID As Guid, ByVal proxy As IClientChannel, ByVal clientOperation As Action)
			'setup the context property for the instance id
            Using scope As New OperationContextScope(CType(proxy, IContextChannel))
                Dim properties As New Dictionary(Of String, String)
                properties.Add("instanceId", workflowID.ToString())
                Dim [property] As New ContextMessageProperty(properties)
                OperationContext.Current.OutgoingMessageProperties.Add("ContextMessageProperty", [property])

                'invoke the client operation for the caller
                clientOperation()
            End Using
		End Sub

		#End Region ' Helper methods for consumers
	End Class
End Namespace
